package PadFile;

################################################################################
# PAD Archiv Demo
# Copyright (c) 2000-2004 by GRAHL software design
#
# http://www.DeuPAD.org
#
# Freeware, Verwendung auf eigene Gefahr.
# Verffentlichung oder Weitergabe nur mit Zustimmung des Autors.
################################################################################
#
# PadFile.pm
#
# Zweck:
#  Ermoeglicht das Einlesen einer PAD Datei von einem entfernten Server ueber
#  HTTP und das Extrahieren von Informationen aus der XML Datei
# Funktionsweise:
#  - Konstruktion mit $PadFile = PadFile->new();
#  - Attribute: padurl:  URL der PAD Datei
#               content: Inhalt der PAD Datei nach Aufruf von readContent()
#               error    Fehlerbeschreibung nach Aufruf von readContent()
#  - Attribute lesen mit z.B. $the_url = $Program->padurl();
#  - Attribute schreiben mit z.B. $Program->padurl($the_url);
#  - $PadFile->readContent() liest die PAD Datei ueber HTTP ein. Rueckgabewert
#    true: das Attribut content enthaelt den Inhalt der gelesenen Datei.
#    Rueckgabewert false: das Attribut error enthaelt eine Fehlerbeschreibung,
#    die Datei konnte nicht gelesen werden.
#  - $PadFile->getValue(<path_list>) liest einen best. Wert aus der PAD Datei
#    z.B. $PadFile->getValue("XML_DIZ_INFO", "Program_Info", "Program_Name")
#
################################################################################

use strict;

use LWP;


##############################################
## Konstruktion                             ##
##############################################
sub new {
  my $self = {};

  $self->{PADURL} = undef;
  $self->{CONTENT} = undef;
  $self->{ERROR} = undef;

  bless($self);
  return $self;
}


##############################################
## Attribute                                ##
##############################################
sub padurl {
  my $self = shift;
  if (@_) { $self->{PADURL} = shift }
  return $self->{PADURL};
}
sub content {
  my $self = shift;
  return $self->{CONTENT};
}
sub error {
  my $self = shift;
  return $self->{ERROR};
}


##############################################
## Methoden                                 ##
##############################################

# Liest die PAD Datei von einem HTTP Server
# Rueckgabewert: true  - kein Fehler $self->content enthaelt den Dateiinhalt
#                false - $self->error enthaelt die Fehlerbeschreibung
sub readContent {
  my $self = shift;
  $self->{ERROR} = undef;

  my $Request = new HTTP::Request(GET => $self->{PADURL});
  my $UserAgent = LWP::UserAgent->new();
  my $Response = $UserAgent->request($Request);
  if ( $Response->is_success ) {
    $self->{CONTENT} = $Response->content;

    # Ermittle XML encoding
    my $encoding = getEncoding($self->{CONTENT});

    # Dekodiere Inhalt
    if ( $encoding eq "UTF-8" ) {
      $self->{CONTENT} =~ s/\xc2//g;
      $self->{CONTENT} =~ s/\xc3(.)/chr(ord($1) + 0x40)/eg;
    }

    # Alternativ kann folgender Code verwendet werden
    # Vorteil: Weitere Encodings neben UTF-8 unterstuetzt
    # Nachteil: Package Encode benoetigt mind. Perl 5.7.3
    #use Encode;
    #if ( $encoding ne "" ) {
    #  eval {
    #    $self->{CONTENT} = Encode::decode($encoding, $self->{CONTENT});
    #  }
    #}

    return 1;
  }
  else {
    $self->{ERROR} = $Response->code . "  " . $Response->message;
    return 0;
  }
}

# Liest ein Feld aus der XML Datei.
# Beispiel: getValue("XML_DIZ_INFO", "Program_Info", "Program_Name")
sub getValue {
  my $self = shift;
  if ( @_ ) {
    my $xml_ = $self->{CONTENT};
    foreach my $tag (@_) {
      $xml_ =~ /(<$tag>)/ || return "";
      $xml_ = $';
    }

    $xml_ =~ /</;
    return $`;
  }
}

# Gibt das encoding aus <?xml version="" encoding="" ?> zurueck
sub getEncoding {
  my $encoding = shift;
  $encoding =~ /<?xml [^>]*encoding=[\'\"]([^\'\"]*)[\'\"][^>]*>.*/ || return "";
  return $1;
}

1;
